﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;

namespace GearHeadSaveDataViewer
{
	public partial class Form1 : Form
	{
		private readonly ListViewItemComparer listViewItemSorter;

		public Form1()
		{
			InitializeComponent();
			this.openFileDialog.InitialDirectory = Program.SavePath;
			this.listViewItemSorter = new ListViewItemComparer(this.nameListView.Sorting);
			this.nameListView.ListViewItemSorter = this.listViewItemSorter;
		}

		protected override void OnFormClosed(FormClosedEventArgs e)
		{
			if (File.Exists(this.openFileDialog.FileName))
			{
				Program.SavePath = Path.GetDirectoryName(this.openFileDialog.FileName);
			}
			base.OnFormClosed(e);
		}

		private void loadButton_Click(object sender, EventArgs e)
		{
			if (this.openFileDialog.ShowDialog() != DialogResult.OK)
			{
				return;
			}
			string save;
			using (System.IO.StreamReader sr = new System.IO.StreamReader(this.openFileDialog.OpenFile(), System.Text.Encoding.GetEncoding("EUC-JP")))
			{
				save = sr.ReadToEnd();
			}
			if (string.IsNullOrEmpty(save) || save.IndexOf("*** GearHead Location Record ***", StringComparison.OrdinalIgnoreCase) < 0)
			{
				MessageBox.Show("このファイルは GearHead のセーブデータではないかもしれません。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			string[] lines = save.Split(new string[1] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
			if (this.nameListView.Items.Count > 0)
			{
				this.nameListView.Items.Clear();
			}
			List<Character> cList = new List<Character>();
			for (int i = 0; i < lines.Length; ++i)
			{
				if (!System.Text.RegularExpressions.Regex.IsMatch(lines[i], @"^0 2 0 [\d]+ [\d]+"))
				{
					continue;
				}
				int scale;
				if (!int.TryParse(lines[i].Split(' ')[4], out scale))
				{
					continue;
				}
				string[] blocks = lines[++i].Split(new char[1] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
				if (blocks.Length != 17)
				{
					continue;
				}
				int reflexes, body, speed, perception, craft, ego, knowledge, charm;
				if (!int.TryParse(blocks[2], out reflexes) || !int.TryParse(blocks[4], out body) || !int.TryParse(blocks[6], out speed) || !int.TryParse(blocks[8], out perception) || !int.TryParse(blocks[10], out craft) || !int.TryParse(blocks[12], out ego) || !int.TryParse(blocks[14], out knowledge) || !int.TryParse(blocks[16], out charm))
				{
					continue;
				}
				string name = string.Empty, job = string.Empty, mecha = string.Empty;
				bool isFemale = false;
				int totalExp = 0, usedExp = 0, money = 0;
				int[] traits = new int[7];
				List<int> talentList = new List<int>();
				List<string> skillLineList = new List<string>();
				List<string> advancedStatusLineList = new List<string>();
				while (lines[++i] != "Z")
				{
					if (lines[i].StartsWith("0 3 0 1", StringComparison.OrdinalIgnoreCase))
					{
						isFemale = true;
					}
					else if (System.Text.RegularExpressions.Regex.IsMatch(lines[i], @"^0 4 0 [\d]+"))
					{
						int.TryParse(lines[i].Split(' ')[3], out totalExp);
					}
					else if (System.Text.RegularExpressions.Regex.IsMatch(lines[i], @"^0 4 1 [\d]+"))
					{
						int.TryParse(lines[i].Split(' ')[3], out usedExp);
					}
					else if (System.Text.RegularExpressions.Regex.IsMatch(lines[i], @"^0 4 2 [\d]+"))
					{
						int.TryParse(lines[i].Split(' ')[3], out money);
					}
					else if (System.Text.RegularExpressions.Regex.IsMatch(lines[i], @"^0 3 -[\d] -?[\d]+"))
					{
						int index = lines[i][5] - '1';
						if (index >= 0 && index < 7 && int.TryParse(lines[i].Split(' ')[3], out traits[index]) && (traits[index] < -100 || traits[index] > 100))
						{
							traits[index] = 0;
						}
					}
					else if (lines[i].StartsWith("0 1 ", StringComparison.OrdinalIgnoreCase))
					{
						skillLineList.Add(lines[i]);
					}
					else if (lines[i].StartsWith("name ", StringComparison.OrdinalIgnoreCase))
					{
						name = lines[i].Split('<')[1].TrimEnd('>');
					}
					else if (lines[i].StartsWith("job", StringComparison.OrdinalIgnoreCase))
					{
						job = lines[i].Split('<')[1].TrimEnd('>');
					}
					else if (lines[i].StartsWith("mecha", StringComparison.OrdinalIgnoreCase))
					{
						mecha = lines[i].Split('<')[1].TrimEnd('>');
					}
					else if (System.Text.RegularExpressions.Regex.IsMatch(lines[i], @"^0 16 [\d]+ 1"))
					{
						int talent;
						if (int.TryParse(lines[i].Split(' ')[2], out talent) && talent > 0 && talent < 29)
						{
							talentList.Add(talent);
						}
					}
					else if (System.Text.RegularExpressions.Regex.IsMatch(lines[i], @"^0 11 [\d]+ [\d]+"))
					{
						advancedStatusLineList.Add(lines[i]);
					}
				}
				cList.Add(new Character(name, Program.ConvertToJapaneseJobName(job), string.IsNullOrEmpty(mecha) ? mecha : mecha.Substring(0, mecha.Length - 4), scale, totalExp, usedExp, money, isFemale, reflexes, body, speed, perception, craft, ego, knowledge, charm, traits[0], traits[1], traits[2], traits[3], traits[4], traits[5], traits[6], talentList.Count > 0 ? talentList.ToArray() : null, skillLineList.ToArray(), advancedStatusLineList.Count > 0 ? advancedStatusLineList.ToArray() : null));
			}
			this.nameListView.Items.AddRange(cList.ToArray());
			this.statusRichTextBox.Clear();
		}

		private void nameListView_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (this.nameListView.SelectedItems.Count > 0)
			{
				this.statusRichTextBox.Clear();
				((Character)this.nameListView.SelectedItems[0]).DrawStatus(this.statusRichTextBox);
			}
		}

		private void nameListView_ColumnClick(object sender, ColumnClickEventArgs e)
		{
			if (this.listViewItemSorter.Column != e.Column)
			{
				this.listViewItemSorter.Column = e.Column;
				this.nameListView.Sort();
			}
		}

		private void ascendingRadioButton_CheckedChanged(object sender, EventArgs e)
		{
			if (this.listViewItemSorter.Sorting != SortOrder.Ascending)
			{
				this.listViewItemSorter.Sorting = SortOrder.Ascending;
				this.nameListView.Sort();
			}
		}

		private void descendingRadioButton_CheckedChanged(object sender, EventArgs e)
		{
			if (this.listViewItemSorter.Sorting != SortOrder.Descending)
			{
				this.listViewItemSorter.Sorting = SortOrder.Descending;
				this.nameListView.Sort();
			}
		}
	}
}
